@@ -71,7 +71,7 @@ if WINDOWS:
 # ------------------------------------------------------------------------------------------------------------
 # Set Version
 
-VERSION = "2.3.0"
+VERSION = "2.3.1"
 
 # ------------------------------------------------------------------------------------------------------------
 # Set TMP
@@ -596,8 +596,8 @@ def countDecimalPoints(step, stepSmall):
         return 2
 
     count = 0
-    value = fmod(abs(stepSmall), 1)
-    while 0.001 < value < 0.99 and count < 6:
+    value = fmod(abs(step), 1)
+    while 0.0001 < value < 0.999 and count < 6:
         value = fmod(value*10, 1)
         count += 1
 
@@ -823,7 +823,7 @@ def getAndSetPath(parent, lineEdit):
 # Backwards-compatible horizontalAdvance/width call, depending on Qt version
 
 def fontMetricsHorizontalAdvance(fontMetrics, string):
-    if QT_VERSION >= 0x51100:
+    if QT_VERSION >= 0x50b00:
         return fontMetrics.horizontalAdvance(string)
     return fontMetrics.width(string)
 
