@@ -77,11 +77,7 @@ public:
         XSetWindowAttributes attr;
         carla_zeroStruct(attr);
 
-        attr.border_pixel = 0;
-        attr.event_mask   = KeyPressMask|KeyReleaseMask;
-
-        if (fIsResizable)
-            attr.event_mask |= StructureNotifyMask;
+        attr.event_mask = KeyPressMask|KeyReleaseMask|FocusChangeMask|StructureNotifyMask|SubstructureNotifyMask;
 
         fHostWindow = XCreateWindow(fDisplay, RootWindow(fDisplay, screen),
                                     0, 0, 300, 300, 0,
@@ -155,10 +151,29 @@ public:
                     XSizeHints hints;
                     carla_zeroStruct(hints);
 
-                    if (XGetNormalHints(fDisplay, childWindow, &hints) && hints.width > 0 && hints.height > 0)
+                    if (XGetNormalHints(fDisplay, childWindow, &hints))
                     {
-                        setSize(static_cast<uint>(hints.width),
-                                static_cast<uint>(hints.height), false);
+                        int width = 0;
+                        int height = 0;
+
+                        if (hints.flags & PSize)
+                        {
+                            width = hints.width;
+                            height = hints.height;
+                        }
+                        else if (hints.flags & PBaseSize)
+                        {
+                            width = hints.base_width;
+                            height = hints.base_height;
+                        }
+                        else if (hints.flags & PMinSize)
+                        {
+                            width = hints.min_width;
+                            height = hints.min_height;
+                        }
+
+                        if (width > 0 && height > 0)
+                            setSize(static_cast<uint>(width), static_cast<uint>(height), false);
                     }
                 }
 
@@ -205,6 +220,9 @@ public:
         // prevent recursion
         if (fIsIdling) return;
 
+        int nextWidth = 0;
+        int nextHeight = 0;
+
         fIsIdling = true;
 
         for (XEvent event; XPending(fDisplay) > 0;)
@@ -223,6 +241,7 @@ public:
                     CARLA_SAFE_ASSERT_CONTINUE(event.xconfigure.width > 0);
                     CARLA_SAFE_ASSERT_CONTINUE(event.xconfigure.height > 0);
 
+                    if (event.xconfigure.window == fHostWindow)
                     {
                         const uint width  = static_cast<uint>(event.xconfigure.width);
                         const uint height = static_cast<uint>(event.xconfigure.height);
@@ -238,9 +257,14 @@ public:
                                 carla_zeroStruct(sizeHints);
 
                                 if (XGetNormalHints(fDisplay, fChildWindow, &sizeHints) && !gErrorTriggered)
+                                {
                                     XSetNormalHints(fDisplay, fHostWindow, &sizeHints);
+                                }
                                 else
+                                {
+                                    carla_stdout("Caught errors while accessing child window");
                                     fChildWindow = 0;
+                                }
 
                                 fChildWindowConfigured = true;
                                 XSetErrorHandler(oldErrorHandler);
@@ -252,6 +276,11 @@ public:
 
                         fCallback->handlePluginUIResized(width, height);
                     }
+                    else if (event.xconfigure.window == fChildWindow && fChildWindow != 0)
+                    {
+                        nextWidth = event.xconfigure.width;
+                        nextHeight = event.xconfigure.height;
+                    }
                     break;
 
             case ClientMessage:
@@ -274,6 +303,13 @@ public:
                     fCallback->handlePluginUIClosed();
                 }
                 break;
+
+            case FocusIn:
+                if (fChildWindow == 0)
+                    fChildWindow = getChildWindow();
+                if (fChildWindow != 0)
+                    XSetInputFocus(fDisplay, fChildWindow, RevertToNone, CurrentTime);
+                break;
             }
 
             if (type != nullptr)
@@ -282,6 +318,18 @@ public:
                 fEventProc(&event);
         }
 
+        if (nextWidth != 0 && nextHeight != 0)
+        {
+            XSizeHints sizeHints;
+            carla_zeroStruct(sizeHints);
+
+            if (XGetNormalHints(fDisplay, fChildWindow, &sizeHints))
+                XSetNormalHints(fDisplay, fHostWindow, &sizeHints);
+
+            XResizeWindow(fDisplay, fHostWindow, static_cast<uint>(nextWidth), static_cast<uint>(nextHeight));
+            XFlush(fDisplay);
+        }
+
         fIsIdling = false;
     }
 
@@ -416,21 +464,12 @@ private:
 #endif
 
 @interface CarlaPluginWindow : NSWindow
-{
-@public
-    CarlaPluginUI::Callback* callback;
-}
-
 - (id) initWithContentRect:(NSRect)contentRect
                  styleMask:(unsigned int)aStyle
                    backing:(NSBackingStoreType)bufferingType
                      defer:(BOOL)flag;
-- (void) setCallback:(CarlaPluginUI::Callback*)cb;
-- (BOOL) acceptsFirstResponder;
 - (BOOL) canBecomeKeyWindow;
 - (BOOL) canBecomeMainWindow;
-- (BOOL) windowShouldClose:(id)sender;
-- (NSSize) windowWillResize:(NSWindow*)sender toSize:(NSSize)frameSize;
 @end
 
 @implementation CarlaPluginWindow
@@ -440,8 +479,6 @@ private:
                   backing:(NSBackingStoreType)bufferingType
                     defer:(BOOL)flag
 {
-    callback = nil;
-
     NSWindow* result = [super initWithContentRect:contentRect
                                         styleMask:aStyle
                                           backing:bufferingType
@@ -455,16 +492,6 @@ private:
     (void)flag;
 }
 
-- (void)setCallback:(CarlaPluginUI::Callback*)cb
-{
-    callback = cb;
-}
-
-- (BOOL)acceptsFirstResponder
-{
-    return YES;
-}
-
 - (BOOL)canBecomeKeyWindow
 {
     return YES;
@@ -475,6 +502,32 @@ private:
     return NO;
 }
 
+@end
+
+@interface CarlaPluginWindowDelegate : NSObject<NSWindowDelegate>
+{
+    CarlaPluginUI::Callback* callback;
+    CarlaPluginWindow* window;
+}
+- (instancetype)initWithWindowAndCallback:(CarlaPluginWindow*)window
+                                 callback:(CarlaPluginUI::Callback*)callback2;
+- (BOOL)windowShouldClose:(id)sender;
+@end
+
+@implementation CarlaPluginWindowDelegate
+
+- (instancetype)initWithWindowAndCallback:(CarlaPluginWindow*)window2
+                                 callback:(CarlaPluginUI::Callback*)callback2
+{
+    if ((self = [super init]))
+    {
+        callback = callback2;
+        window = window2;
+    }
+
+  return self;
+}
+
 - (BOOL)windowShouldClose:(id)sender
 {
     if (callback != nil)
@@ -486,15 +539,25 @@ private:
     (void)sender;
 }
 
-- (NSSize) windowWillResize:(NSWindow*)sender toSize:(NSSize)frameSize
+@end
+
+@interface CarlaPluginView : NSView
+- (void)resizeWithOldSuperviewSize:(NSSize)oldSize;
+@end
+
+@implementation CarlaPluginView
+
+- (void)resizeWithOldSuperviewSize:(NSSize)oldSize
 {
-    if (callback != nil)
-        callback->handlePluginUIResized(frameSize.width, frameSize.height);
+    [super resizeWithOldSuperviewSize:oldSize];
 
-    return frameSize;
-
-    // unused
-    (void)sender;
+    /*
+    for (NSView* subview in [self subviews])
+    {
+        [subview setFrame:NSMakeRect(0, 0, oldSize.width, oldSize.height)];
+        break;
+    }
+    */
 }
 
 @end
@@ -502,17 +565,17 @@ private:
 class CocoaPluginUI : public CarlaPluginUI
 {
 public:
-    CocoaPluginUI(Callback* const cb, const uintptr_t parentId, const bool isResizable) noexcept
-        : CarlaPluginUI(cb, isResizable),
+    CocoaPluginUI(Callback* const callback, const uintptr_t parentId, const bool isResizable) noexcept
+        : CarlaPluginUI(callback, isResizable),
           fView(nullptr),
           fParentWindow(nullptr),
           fWindow(nullptr)
     {
-        carla_debug("CocoaPluginUI::CocoaPluginUI(%p, " P_UINTPTR, "%s)", cb, parentId, bool2str(isResizable));
+        carla_debug("CocoaPluginUI::CocoaPluginUI(%p, " P_UINTPTR, "%s)", callback, parentId, bool2str(isResizable));
         const CarlaBackend::AutoNSAutoreleasePool arp;
         [NSApplication sharedApplication];
 
-        fView = [[NSView new]retain];
+        fView = [[CarlaPluginView new]retain];
         CARLA_SAFE_ASSERT_RETURN(fView != nullptr,)
 
         uint style = NSClosableWindowMask | NSTitledWindowMask;
@@ -521,7 +584,7 @@ public:
             style |= NSResizableWindowMask;
         */
 
-        NSRect frame = NSMakeRect(0, 0, 100, 100);
+        const NSRect frame = NSMakeRect(0, 0, 100, 100);
 
         fWindow = [[[CarlaPluginWindow alloc]
             initWithContentRect:frame
@@ -537,13 +600,16 @@ public:
             return;
         }
 
-        //if (! isResizable)
+        ((NSWindow*)fWindow).delegate = [[[CarlaPluginWindowDelegate alloc] 
+            initWithWindowAndCallback:fWindow
+                             callback:callback] retain];
+
+        // if (! isResizable)
         {
             [fView setAutoresizingMask:NSViewNotSizable];
             [[fWindow standardWindowButton:NSWindowZoomButton] setHidden:YES];
         }
 
-        [fWindow setCallback:cb];
         [fWindow setContentView:fView];
         [fWindow makeFirstResponder:fView];
 
@@ -574,7 +640,7 @@ public:
         if (fParentWindow != nullptr)
         {
             [fParentWindow addChildWindow:fWindow
-                                   ordered:NSWindowAbove];
+                                  ordered:NSWindowAbove];
         }
         else
         {
@@ -604,6 +670,7 @@ public:
         CARLA_SAFE_ASSERT_RETURN(fWindow != nullptr,);
 
         [fWindow makeKeyAndOrderFront:fWindow];
+        [fWindow orderFrontRegardless];
         [NSApp activateIgnoringOtherApps:YES];
     }
 
@@ -672,10 +739,17 @@ public:
                                    ordered:NSWindowAbove];
     }
 
-    void setChildWindow(void* const window) override
+    void setChildWindow(void* const childWindow) override
     {
-        carla_debug("CocoaPluginUI::setChildWindow(%p)", window);
-        CARLA_SAFE_ASSERT_RETURN(window != nullptr,);
+        carla_debug("CocoaPluginUI::setChildWindow(%p)", childWindow);
+        CARLA_SAFE_ASSERT_RETURN(childWindow != nullptr,);
+
+        NSView* const view = (NSView*)childWindow;
+        const NSRect frame = [view frame];
+
+        [fWindow setContentSize:frame.size];
+        [fView setFrame:frame];
+        [fView setNeedsDisplay:YES];
     }
 
     void* getPtr() const noexcept override
@@ -705,7 +779,7 @@ private:
 
 #ifdef CARLA_OS_WIN
 
-#define PUGL_LOCAL_CLOSE_MSG (WM_USER + 50)
+#define CARLA_LOCAL_CLOSE_MSG (WM_USER + 50)
 
 static LRESULT CALLBACK wndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);
 
@@ -745,10 +819,7 @@ public:
         int winFlags = WS_POPUPWINDOW | WS_CAPTION;
 
         if (isResizable)
-        {
-            // not supported right now
-            // winFlags |= WS_SIZEBOX;
-        }
+            winFlags |= WS_SIZEBOX;
 
 #ifdef BUILDING_CARLA_FOR_WINE
         const uint winType = WS_EX_DLGMODALFRAME;
@@ -764,7 +835,8 @@ public:
                                  parent, nullptr,
                                  hInstance, nullptr);
 
-        if (fWindow == nullptr) {
+        if (fWindow == nullptr)
+        {
             const DWORD errorCode = ::GetLastError();
             carla_stderr2("CreateWindowEx failed with error code 0x%x, class name was '%s'",
                           errorCode, fWindowClass.lpszClassName);
@@ -812,6 +884,9 @@ public:
             fFirstShow = false;
             RECT rectChild, rectParent;
 
+            if (fChildWindow != nullptr && GetWindowRect(fChildWindow, &rectChild))
+                setSize(rectChild.right - rectChild.left, rectChild.bottom - rectChild.top, false);
+
             if (fParentWindow != nullptr &&
                 GetWindowRect(fWindow, &rectChild) &&
                 GetWindowRect(fParentWindow, &rectParent))
@@ -857,7 +932,7 @@ public:
             switch (msg.message)
             {
             case WM_QUIT:
-            case PUGL_LOCAL_CLOSE_MSG:
+            case CARLA_LOCAL_CLOSE_MSG:
                 fIsVisible = false;
                 CARLA_SAFE_ASSERT_BREAK(fCallback != nullptr);
                 fCallback->handlePluginUIClosed();
@@ -876,12 +951,21 @@ public:
         {
             switch (message)
             {
+            case WM_SIZE:
+                if (fChildWindow != nullptr)
+                {
+                    RECT rect;
+                    GetClientRect(fWindow, &rect);
+                    SetWindowPos(fChildWindow, 0, 0, 0, rect.right, rect.bottom,
+                                 SWP_NOACTIVATE|SWP_NOMOVE|SWP_NOOWNERZORDER|SWP_NOZORDER);
+                }
+                break;
             case WM_QUIT:
-            case PUGL_LOCAL_CLOSE_MSG:
-                    fIsVisible = false;
-                    CARLA_SAFE_ASSERT_BREAK(fCallback != nullptr);
-                    fCallback->handlePluginUIClosed();
-                    break;
+            case CARLA_LOCAL_CLOSE_MSG:
+                fIsVisible = false;
+                CARLA_SAFE_ASSERT_BREAK(fCallback != nullptr);
+                fCallback->handlePluginUIClosed();
+                break;
             }
         }
 
@@ -931,7 +1015,7 @@ public:
     {
         CARLA_SAFE_ASSERT_RETURN(winId != nullptr,);
 
-        fChildWindow = (HWND)fChildWindow;
+        fChildWindow = (HWND)winId;
     }
 
     void* getPtr() const noexcept override
@@ -961,7 +1045,7 @@ LRESULT CALLBACK wndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
     switch (message)
     {
     case WM_CLOSE:
-        PostMessage(hwnd, PUGL_LOCAL_CLOSE_MSG, wParam, lParam);
+        PostMessage(hwnd, CARLA_LOCAL_CLOSE_MSG, wParam, lParam);
         return 0;
 
 #if 0
